/*
 * Copyright (c) 2012-2013 CNRS/LAAS
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/**
 * niutKalman.cc
 * Kalman filtering class
 **/
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "niutKalman.h"

using namespace std;

// initial values
float P_last[3][25][3] =  {0.1};

// Noises
float Q = 0.01; // Process noise
float R = 0.4; // Observation noise

float K;
float P[3][25][3];
float P_temp;
float xhat_temp;
float xhat;
float xhat_last[3][25][3] = {0}; // initialize


float
Kalman::filter(float z_meas, int id, int joint, int coord)
{
    // prediction
    xhat_temp = xhat_last[id][joint][coord];
    P_temp = P_last[id][joint][coord] + Q;

    // calculate the Kalman gain
    K = P_temp * (1./(P_temp + R));

    // update
    xhat = xhat_temp + K * (z_meas - xhat_temp);
    P[id][joint][coord] = (1. - K) * P_temp;

    //update our last's
    P_last[id][joint][coord] = P[id][joint][coord];
    xhat_last[id][joint][coord] = xhat;

    return xhat;
}
